@@ -71,7 +71,7 @@ class TestVariable(unittest.TestCase):
         cls.var = Variable("x")
 
     def test_name(self):
-        self.assertEqual(repr(self.var), "Variable('x')")
+        self.assertEqual(repr(self.var), "Variable(name='x')")
 
     def test_to_val(self):
         string_var = StringVariable("x")
@@ -199,23 +199,23 @@ class TestDiscreteVariable(VariableTest):
         var = DiscreteVariable.make("a", values=["F", "M"])
         self.assertEqual(
             repr(var),
-            "DiscreteVariable('a', values=['F', 'M'])")
+            "DiscreteVariable(name='a', values=['F', 'M'])")
         var.base_value = 1
         self.assertEqual(
             repr(var),
-            "DiscreteVariable('a', values=['F', 'M'], base_value=1)")
+            "DiscreteVariable(name='a', values=['F', 'M'], base_value=1)")
         var.ordered = True
         self.assertEqual(
             repr(var),
-            "DiscreteVariable('a', values=['F', 'M'], "
+            "DiscreteVariable(name='a', values=['F', 'M'], "
             "ordered=True, base_value=1)")
 
         var = DiscreteVariable.make("a", values="1234567")
         self.assertEqual(
             repr(var),
-            "DiscreteVariable('a', values=['1', '2', '3', '4', '5', ...])")
+            "DiscreteVariable(name='a', values=['1', '2', '3', '4', '5', '6', '7'])")
 
-    @unittest.skipUnless(is_on_path("PyQt4"), "PyQt4 is not importable")
+    @unittest.skipUnless(is_on_path("PyQt4") or is_on_path("PyQt5"), "PyQt is not importable")
     def test_colors(self):
         var = DiscreteVariable.make("a", values=["F", "M"])
         self.assertIsNone(var._colors)
@@ -236,6 +236,17 @@ class TestDiscreteVariable(VariableTest):
         var.attributes["colors"] = ['#0a0b0c', '#0d0e0f']
         np.testing.assert_almost_equal(var.colors, [[10, 11, 12], [13, 14, 15]])
 
+        # Test ncolors adapts to nvalues
+        var = DiscreteVariable.make('foo', values=['d', 'r'])
+        self.assertEqual(len(var.colors), 2)
+        var.add_value('e')
+        self.assertEqual(len(var.colors), 3)
+        user_defined = (0, 0, 0)
+        var.set_color(2, user_defined)
+        var.add_value('k')
+        self.assertEqual(len(var.colors), 4)
+        np.testing.assert_array_equal(var.colors[2], user_defined)
+
 
 @variabletest(ContinuousVariable)
 class TestContinuousVariable(VariableTest):
