@@ -142,13 +142,14 @@ class SettingProvider:
         self._initialize_providers(instance, data)
 
     def _initialize_settings(self, instance, data):
+        if data is None:
+            data = {}
         for name, setting in self.settings.items():
-            if data and name in data:
-                setattr(instance, name, data[name])
-            elif isinstance(setting.default, _IMMUTABLES):
-                setattr(instance, name, setting.default)
+            value = data.get(name, setting.default)
+            if isinstance(value, _IMMUTABLES):
+                setattr(instance, name, value)
             else:
-                setattr(instance, name, copy.copy(setting.default))
+                setattr(instance, name, copy.copy(value))
 
     def _initialize_providers(self, instance, data):
         if not data:
@@ -987,9 +988,8 @@ class DomainContextHandler(ContextHandler):
             setattr(widget, self.reservoir, ll)
 
     def encode_setting(self, context, setting, value):
-        value = copy.copy(value)
         if isinstance(value, list):
-            return value
+            return copy.copy(value)
         elif isinstance(setting, ContextSetting):
             if isinstance(value, str):
                 if not setting.exclude_attributes and value in context.attributes:
@@ -998,7 +998,7 @@ class DomainContextHandler(ContextHandler):
                     return value, context.metas[value]
             elif isinstance(value, Variable):
                 return value.name, 100 + vartype(value)
-        return value, -2
+        return copy.copy(value), -2
 
     def decode_setting(self, setting, value, domain=None):
         if isinstance(value, tuple):
