@@ -1,26 +1,35 @@
 import os
+import time
+from contextlib import contextmanager
 import unittest
+from unittest.mock import Mock
+
+import sip
 
 import numpy as np
-
-from Orange.base import SklLearner, SklModel
+from AnyQt.QtCore import Qt
+from AnyQt.QtTest import QTest
 from AnyQt.QtWidgets import (
     QApplication, QComboBox, QSpinBox, QDoubleSpinBox, QSlider
 )
-import sip
 
-from Orange.data import Table
+from Orange.base import SklModel, Model
+from Orange.canvas.report.owreport import OWReport
+from Orange.classification.base_classification import (
+    LearnerClassification, ModelClassification
+)
+from Orange.data import Table, Domain, DiscreteVariable, ContinuousVariable
+from Orange.modelling import Fitter
 from Orange.preprocess import RemoveNaNColumns, Randomize
 from Orange.preprocess.preprocess import PreprocessorList
-from Orange.classification.base_classification import (LearnerClassification,
-                                                       ModelClassification)
-from Orange.regression.base_regression import LearnerRegression, ModelRegression
-from Orange.canvas.report.owreport import OWReport
+from Orange.regression.base_regression import (
+    LearnerRegression, ModelRegression
+)
+from Orange.widgets.utils.annotated_data import (
+    ANNOTATED_DATA_FEATURE_NAME, ANNOTATED_DATA_SIGNAL_NAME
+)
 from Orange.widgets.utils.owlearnerwidget import OWBaseLearner
-from Orange.widgets.utils.annotated_data import (ANNOTATED_DATA_FEATURE_NAME,
-                                                 ANNOTATED_DATA_SIGNAL_NAME)
 
-# For tests, let memory freeing entirely to Python / OS
 sip.setdestroyonexit(False)
 
 app = None
@@ -55,9 +64,8 @@ class WidgetTest(GuiTest):
 
     Contains helper methods widget creation and working with signals.
 
-    All widgets should be created by the create_widget method, as it
-    remembers created widgets and properly destroys them in tearDownClass
-    to avoid segmentation faults when QApplication gets destroyed.
+    All widgets should be created by the create_widget method, as this
+    will ensure they are created correctly.
     """
 
     #: list[OwWidget]
@@ -67,9 +75,8 @@ class WidgetTest(GuiTest):
     def setUpClass(cls):
         """Prepare environment for test execution
 
-        Prepare a list for tracking created widgets and construct a
-        dummy signal manager. Monkey patch OWReport.get_instance to
-        return a manually_created instance.
+        Construct a dummy signal manager and monkey patch
+        OWReport.get_instance to return a manually created instance.
         """
         super().setUpClass()
 
@@ -82,7 +89,13 @@ class WidgetTest(GuiTest):
         OWReport.get_instance = lambda: report
 
     def create_widget(self, cls, stored_settings=None, reset_default_settings=True):
-        """Create a widget instance.
+        """Create a widget instance using mock signal_manager.
+
+        When used with default parameters, it also overrides settings stored
+        on disk with default defined in class.
+
+        After widget is created, QApplication.process_events is called to
+        allow any singleShot timers defined in __init__ to execute.
 
         Parameters
         ----------
@@ -129,12 +142,34 @@ class WidgetTest(GuiTest):
             settings_handler.global_contexts = []
 
     @staticmethod
-    def process_events():
-        """Process Qt events.
+    def process_events(until: callable=None):
+        """Process Qt events, optionally until `until` returns
+        something True-ish.
 
         Needs to be called manually as QApplication.exec is never called.
+
+        Parameters
+        ----------
+        until: callable or None
+            If callable, the events are processed until the function returns
+            something True-ish.
+
+        Returns
+        -------
+        If until is not None, the True-ish result of its call.
         """
-        app.processEvents()
+        if until is None:
+            until = lambda: True
+
+        while True:
+            app.processEvents()
+            try:
+                result = until()
+                if result:
+                    return result
+            except Exception:  # until can fail with anything; pylint: disable=broad-except
+                pass
+            time.sleep(.05)
 
     def show(self, widget=None):
         """Show widget in interactive mode.
@@ -185,6 +220,24 @@ class WidgetTest(GuiTest):
             widget = self.widget
         return self.signal_manager.outputs.get((widget, output_name), None)
 
+    @contextmanager
+    def modifiers(self, modifiers):
+        """
+        Context that simulates pressed modifiers
+
+        Since QTest.keypress requries pressing some key, we simulate
+        pressing "BassBoost" that looks exotic enough to not meddle with
+        anything.
+        """
+        old_modifiers = QApplication.keyboardModifiers()
+        try:
+            QTest.keyPress(self.widget, Qt.Key_BassBoost, modifiers)
+            yield
+        finally:
+            QTest.keyRelease(self.widget, Qt.Key_BassBoost, old_modifiers)
+
+
+
 
 class BaseParameterMapping:
     """Base class for mapping between gui components and learner's parameters
@@ -208,12 +261,20 @@ class BaseParameterMapping:
         It sets component's value.
     """
 
-    def __init__(self, name, gui_element, values, getter, setter):
+    def __init__(self, name, gui_element, values, getter, setter,
+                 problem_type="both"):
         self.name = name
         self.gui_element = gui_element
         self.values = values
         self.get_value = getter
         self.set_value = setter
+        self.problem_type = problem_type
+
+    def __str__(self):
+        if self.problem_type == "both":
+            return self.name
+        else:
+            return "%s (%s)" % (self.name, self.problem_type)
 
 
 class DefaultParameterMapping(BaseParameterMapping):
@@ -261,11 +322,13 @@ class ParameterMapping(BaseParameterMapping):
     """
 
     def __init__(self, name, gui_element, values=None,
-                 getter=None, setter=None):
-        super().__init__(name, gui_element,
-                         values or self._default_values(gui_element),
-                         getter or self._default_get_value(gui_element, values),
-                         setter or self._default_set_value(gui_element, values))
+                 getter=None, setter=None, **kwargs):
+        super().__init__(
+            name, gui_element,
+            values or self._default_values(gui_element),
+            getter or self._default_get_value(gui_element, values),
+            setter or self._default_set_value(gui_element, values),
+            **kwargs)
 
     @staticmethod
     def get_gui_element(widget, attribute):
@@ -320,20 +383,31 @@ class WidgetLearnerTestMixin:
     widget = None  # type: OWBaseLearner
 
     def init(self):
-        self.iris = Table("iris")
-        self.housing = Table("housing")
-        if issubclass(self.widget.LEARNER, LearnerClassification):
-            self.data = self.iris
-            self.inadequate_data = self.housing
+        cls_ds = Table(datasets.path("testing_dataset_cls"))
+        reg_ds = Table(datasets.path("testing_dataset_reg"))
+
+        if issubclass(self.widget.LEARNER, Fitter):
+            self.data = cls_ds
+            self.valid_datasets = (cls_ds, reg_ds)
+            self.inadequate_dataset = ()
+            self.learner_class = Fitter
+            self.model_class = Model
+            self.model_name = 'Model'
+        elif issubclass(self.widget.LEARNER, LearnerClassification):
+            self.data = cls_ds
+            self.valid_datasets = (cls_ds,)
+            self.inadequate_dataset = (reg_ds,)
             self.learner_class = LearnerClassification
-            self.model_name = "Classifier"
             self.model_class = ModelClassification
+            self.model_name = 'Classifier'
         else:
-            self.data = self.housing
-            self.inadequate_data = self.iris
+            self.data = reg_ds
+            self.valid_datasets = (reg_ds,)
+            self.inadequate_dataset = (cls_ds,)
             self.learner_class = LearnerRegression
-            self.model_name = "Predictor"
             self.model_class = ModelRegression
+            self.model_name = 'Predictor'
+
         self.parameters = []
 
     def test_has_unconditional_apply(self):
@@ -356,36 +430,44 @@ class WidgetLearnerTestMixin:
 
     def test_input_data_learner_adequacy(self):
         """Check if error message is shown with inadequate data on input"""
-        self.send_signal("Data", self.inadequate_data)
-        self.widget.apply_button.button.click()
-        self.assertTrue(self.widget.Error.data_error.is_shown())
-        self.send_signal("Data", self.data)
-        self.assertFalse(self.widget.Error.data_error.is_shown())
+        for inadequate in self.inadequate_dataset:
+            self.send_signal("Data", inadequate)
+            self.widget.apply_button.button.click()
+            self.assertTrue(self.widget.Error.data_error.is_shown())
+        for valid in self.valid_datasets:
+            self.send_signal("Data", valid)
+            self.assertFalse(self.widget.Error.data_error.is_shown())
 
     def test_input_preprocessor(self):
         """Check learner's preprocessors with an extra pp on input"""
-        self.assertIsNone(self.widget.preprocessors)
         self.send_signal("Preprocessor", Randomize)
-        pp = (Randomize,) + tuple(self.widget.LEARNER.preprocessors)
-        self.assertEqual(pp, self.widget.preprocessors)
+        self.assertEqual(
+            Randomize, self.widget.preprocessors,
+            'Preprocessor not added to widget preprocessors')
         self.widget.apply_button.button.click()
-        self.assertEqual(pp, tuple(self.widget.learner.preprocessors))
+        self.assertEqual(
+            (Randomize,), self.widget.learner.preprocessors,
+            'Preprocessors were not passed to the learner')
 
     def test_input_preprocessors(self):
         """Check multiple preprocessors on input"""
         pp_list = PreprocessorList([Randomize, RemoveNaNColumns])
         self.send_signal("Preprocessor", pp_list)
         self.widget.apply_button.button.click()
-        self.assertEqual([pp_list] + list(self.widget.LEARNER.preprocessors),
-                         self.widget.learner.preprocessors)
+        self.assertEqual(
+            (pp_list,), self.widget.learner.preprocessors,
+            '`PreprocessorList` was not added to preprocessors')
 
     def test_input_preprocessor_disconnect(self):
         """Check learner's preprocessors after disconnecting pp from input"""
         self.send_signal("Preprocessor", Randomize)
-        self.assertIsNotNone(self.widget.preprocessors)
+        self.widget.apply_button.button.click()
+        self.assertEqual(Randomize, self.widget.preprocessors)
+
         self.send_signal("Preprocessor", None)
-        self.assertEqual(self.widget.preprocessors,
-                         tuple(self.widget.LEARNER.preprocessors))
+        self.widget.apply_button.button.click()
+        self.assertIsNone(self.widget.preprocessors,
+                          'Preprocessors not removed on disconnect.')
 
     def test_output_learner(self):
         """Check if learner is on output after apply"""
@@ -403,7 +485,7 @@ class WidgetLearnerTestMixin:
         self.assertIsNone(self.get_output(self.model_name))
         self.widget.apply_button.button.click()
         self.assertIsNone(self.get_output(self.model_name))
-        self.send_signal("Data", self.data)
+        self.send_signal('Data', self.data)
         self.widget.apply_button.button.click()
         model = self.get_output(self.model_name)
         self.assertIsNotNone(model)
@@ -428,48 +510,101 @@ class WidgetLearnerTestMixin:
         self.widget.apply_button.button.click()
         self.assertEqual(self.get_output(self.model_name).name, new_name)
 
+    def _get_param_value(self, learner, param):
+        if isinstance(learner, Fitter):
+            # Both is just a was to indicate to the tests, fitters don't
+            # actually support this
+            if param.problem_type == "both":
+                problem_type = learner.CLASSIFICATION
+            else:
+                problem_type = param.problem_type
+            return learner.get_params(problem_type).get(param.name)
+        else:
+            return learner.params.get(param.name)
+
     def test_parameters_default(self):
-        """Check if learner's parameters are set to default (widget's) values"""
-        self.widget.apply_button.button.click()
-        if hasattr(self.widget.learner, "params"):
-            learner_params = self.widget.learner.params
+        """Check if learner's parameters are set to default (widget's) values
+        """
+        for dataset in self.valid_datasets:
+            self.send_signal("Data", dataset)
+            self.widget.apply_button.button.click()
             for parameter in self.parameters:
-                self.assertEqual(learner_params.get(parameter.name),
-                                 parameter.get_value())
+                # Skip if the param isn't used for the given data type
+                if self._should_check_parameter(parameter, dataset):
+                    self.assertEqual(
+                        self._get_param_value(self.widget.learner, parameter),
+                        parameter.get_value())
 
     def test_parameters(self):
-        def get_value(learner, name):
-            # Handle SKL and skl-like learners, and non-SKL learners
-            if hasattr(learner, "params"):
-                return learner.params.get(name)
-            else:
-                return getattr(learner, name)
-
         """Check learner and model for various values of all parameters"""
-        for parameter in self.parameters:
-            assert isinstance(parameter, BaseParameterMapping)
-            for value in parameter.values:
-                self.send_signal("Data", self.data)
-                parameter.set_value(value)
-                self.widget.apply_button.button.click()
-                param = get_value(self.widget.learner, parameter.name)
-                self.assertEqual(param, parameter.get_value(),
-                                 "Mismatching setting for parameter '{}'".
-                                 format(parameter.name))
-                self.assertEqual(param, value,
-                                 "Mismatching setting for parameter '{}'".
-                                 format(parameter.name))
-                param = get_value(self.get_output("Learner"), parameter.name)
-                self.assertEqual(param, value,
-                                 "Mismatching setting for parameter '{}'".
-                                 format(parameter.name))
-                if issubclass(self.widget.LEARNER, SklModel):
-                    model = self.get_output(self.model_name)
-                    if model is not None:
-                        self.assertEqual(get_value(model, parameter.name), value)
-                        self.assertFalse(self.widget.Error.active)
-                    else:
-                        self.assertTrue(self.widget.Error.active)
+        # Test params on every valid dataset, since some attributes may apply
+        # to only certain problem types
+        for dataset in self.valid_datasets:
+            self.send_signal("Data", dataset)
+
+            for parameter in self.parameters:
+                # Skip if the param isn't used for the given data type
+                if not self._should_check_parameter(parameter, dataset):
+                    continue
+
+                assert isinstance(parameter, BaseParameterMapping)
+
+                for value in parameter.values:
+                    parameter.set_value(value)
+                    self.widget.apply_button.button.click()
+                    param = self._get_param_value(self.widget.learner, parameter)
+                    self.assertEqual(
+                        param, parameter.get_value(),
+                        "Mismatching setting for parameter '%s'" % parameter)
+                    self.assertEqual(
+                        param, value,
+                        "Mismatching setting for parameter '%s'" % parameter)
+                    param = self._get_param_value(self.get_output("Learner"), parameter)
+                    self.assertEqual(
+                        param, value,
+                        "Mismatching setting for parameter '%s'" % parameter)
+
+                    if issubclass(self.widget.LEARNER, SklModel):
+                        model = self.get_output(self.model_name)
+                        if model is not None:
+                            self.assertEqual(self._get_param_value(model, parameter), value)
+                            self.assertFalse(self.widget.Error.active)
+                        else:
+                            self.assertTrue(self.widget.Error.active)
+
+    def test_params_trigger_settings_changed(self):
+        """Check that the learner gets updated whenever a param is changed."""
+        for dataset in self.valid_datasets:
+            self.send_signal("Data", dataset)
+
+            for parameter in self.parameters:
+                # Skip if the param isn't used for the given data type
+                if not self._should_check_parameter(parameter, dataset):
+                    continue
+
+                assert isinstance(parameter, BaseParameterMapping)
+                # Set the mock here so we can include the param name in the
+                # error message, so if any test fails, we see where
+                # We mock `apply` and not `settings_changed` since that's
+                # sometimes connected with Qt signals, which are not directly
+                # called
+                self.widget.apply = Mock(name="apply(%s)" % parameter)
+                # Since the settings only get updated when the value actually
+                # changes, find a value that isn't the same as the current
+                # value and try with that
+                new_value = [x for x in parameter.values
+                             if x != parameter.get_value()][0]
+                parameter.set_value(new_value)
+                self.widget.apply.assert_called_once_with()
+
+    @staticmethod
+    def _should_check_parameter(parameter, data):
+        """Should the param be passed into the learner given the data"""
+        return ((parameter.problem_type == "classification" and
+                 data.domain.has_discrete_class) or
+                (parameter.problem_type == "regression" and
+                 data.domain.has_continuous_class) or
+                (parameter.problem_type == "both"))
 
 
 class WidgetOutputsTestMixin:
@@ -595,3 +730,26 @@ class datasets:
         data : Orange.data.Table
         """
         return Table(cls.path("missing_data_3.tab"))
+
+    @classmethod
+    def data_one_column_nans(cls):
+        """
+        Data set with two continuous features and one discrete. One continuous
+        columns has missing values (NaN).
+
+        Returns
+        -------
+        data : Orange.data.Table
+        """
+        table = Table(
+            Domain(
+                [ContinuousVariable("a"),
+                 ContinuousVariable("b"),
+                 DiscreteVariable("c", values=["y", "n"])]
+            ),
+            list(zip(
+                [42.48, 16.84, 15.23, 23.8],
+                ["", "", "", ""],
+                "ynyn"
+            )))
+        return table
