@@ -5,9 +5,9 @@ import numpy as np
 from AnyQt.QtCore import QEvent, QPoint, Qt
 from AnyQt.QtGui import QMouseEvent
 
-from Orange.data import Table, DiscreteVariable, Domain, ContinuousVariable
+from Orange.data import Table, DiscreteVariable, Domain, ContinuousVariable, StringVariable
 from Orange.widgets.tests.base import WidgetTest, WidgetOutputsTestMixin
-from Orange.widgets.visualize.owmosaic import OWMosaicDisplay, MosaicVizRank
+from Orange.widgets.visualize.owmosaic import OWMosaicDisplay
 
 
 class TestOWMosaicDisplay(WidgetTest, WidgetOutputsTestMixin):
@@ -26,6 +26,10 @@ class TestOWMosaicDisplay(WidgetTest, WidgetOutputsTestMixin):
         """Check that the widget doesn't crash on empty data"""
         self.send_signal("Data", self.data[:0])
 
+    def test_empty_column(self):
+        """Check that the widget doesn't crash if the columns are empty"""
+        self.send_signal("Data", self.data[:, :0])
+
     def _select_data(self):
         self.widget.select_area(1, QMouseEvent(
             QEvent.MouseButtonPress, QPoint(), Qt.LeftButton,
@@ -40,6 +44,13 @@ class TestOWMosaicDisplay(WidgetTest, WidgetOutputsTestMixin):
                      metas=np.arange(6).reshape(6, 1))
         self.send_signal("Data", data)
 
+    def test_string_meta(self):
+        """Check widget for dataset with only one string meta"""
+        domain = Domain([], metas=[StringVariable("m")])
+        data = Table(domain, np.empty((6, 0)),
+                     metas=np.array(["meta"] * 6).reshape(6, 1))
+        self.send_signal("Data", data)
+
     def test_missing_values(self):
         """Check widget for dataset with missing values"""
         data = Table(Domain([DiscreteVariable("c1", [])]),
@@ -53,6 +64,14 @@ class TestOWMosaicDisplay(WidgetTest, WidgetOutputsTestMixin):
         data = Table(Domain(attrs, class_var), X, np.array([np.nan] * 6))
         self.send_signal("Data", data)
 
+    def test_keyerror(self):
+        """gh-2014
+        Check if it works when a table has only one row or duplicates.
+        Discretizer must have remove_const set to False.
+        """
+        data = Table("iris")
+        data = data[0:1]
+        self.send_signal("Data", data)
 
 # Derive from WidgetTest to simplify creation of the Mosaic widget, although
 # we are actually testing the MosaicVizRank dialog and not the widget
@@ -186,3 +205,21 @@ class MosaicVizRankTests(WidgetTest):
         data = Table("housing.tab")
         widget.set_data(data)
         vizrank.toggle()
+
+    def test_nan_column(self):
+        """
+        A column with only NaN-s used to throw an error
+        (ZeroDivisionError) when loaded into widget.
+        GH-2046
+        """
+        table = Table(
+            Domain(
+                [ContinuousVariable("a"), ContinuousVariable("b"), ContinuousVariable("c")]),
+            np.array([
+                [0, np.NaN, 0],
+                [0, np.NaN, 0],
+                [0, np.NaN, 0]
+            ])
+        )
+        self.send_signal("Data", table)
+
